//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef GAME_SCENARIO_H
#define GAME_SCENARIO_H

#include <paragui.h>
#include <pgeventobject.h>
#include <pgmessageobject.h>
#include <string>
#include <sigc++/object_slot.h>
#include "player.h"

class Server;
class Client;
class GamePreferencesDialog;

class GameScenario : public PG_EventObject, public PG_MessageObject,
                     public SigC::Object
{
	public:
		enum GameType {SCENARIO = 1, SINGLE_PLAYER = 2, MULTI_PLAYER_HOST = 3,
            MULTI_PLAYER_CLIENT = 4};
		enum MapType {RANDOM, LOADED};

		// CREATORS
		GameScenario(GameType gametype, std::string ip, int port);
        GameScenario(std::string savegame, bool& broken);
		~GameScenario();

		// MANIPULATORS
		void setMapFile(std::string mapFile);
		void setName(std::string name);
	
		// ACCESSORS
		int getTurns() const {return d_turns;}
		GameType getGameType() const {return d_gameType;}
		std::string getName() const {return d_name;}
		bool isMultiPlayer() const;
		
		void nextTurn();

        //  save/load
		bool saveGame(std::string filename); // filename with path
        bool load(std::string tag, XML_Helper* helper);

	private:
		// MANIPULATORS
		void initGame();
		void initSinglePlayer();
		void initScenario();
		void initMultiPlayer();
		
		void addPlayer(std::string name, std::string armyset,
					SDL_Color color, Player::TYPE type);
		void checkInitialisation();
		void produceFirstArmy();
		void assignPlayers();
		void misc();
		void cleanup();

		// DATA
        GamePreferencesDialog* d_gamePreferencesDialog;
		GameType d_gameType;
		int d_turns;
		std::string d_name;
        Server* d_server;
        Client* d_client;
};

#endif // GAME_SCENARIO_H

// End of file
