//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef MAPGENERATOR_H
#define MAPGENERATOR_H

#include <string>
#include <vector>

class MapGenerator
{
    public :
        MapGenerator();
        ~MapGenerator();

        //for setting start parametres
        int setNoCities(int nocities);
        int setNoRuins(int noruins);
        int setNoTemples(int notemples);

		void setPercentages(int pwater, int pforest, int pswamp,
                            int phills, int pmountains);

        //getters
        int getNoCities() {return d_nocities;}
        int getNoRuins() {return d_noruins;}
        int getNoTemples() {return d_notemples;}
		
        //for creating the map
        void MakeMap(int xsize,int ysize);
        void DumpMap(const char * filename);

        //getters
        char* getMap(int& xsize, int& ysize);
        char* getBuildings(int& xsize, int& ysize);
        std::vector<std::string>* getCitynames();

    private :
        void makePlains();
        void makeTerrain(char c,int percent);
        void makeHills(char c,int percent);
        void makeCities(int cities);
        void makeBuildings(char c, int building);
        bool canPutBuilding(int x,int y);

        //Data
        char d_terrain[100][100];
        char d_building[100][100];
        int d_xsize;
        int d_ysize;
        int d_pswamp, d_pwater, d_pforest, d_phills, d_pmountains;
        unsigned int d_nocities, d_notemples, d_noruins;
        std::vector<std::string> d_citynames, d_namelist;
};

#endif

// End of file
