//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "cityinfo.h"
#include <pgmessagebox.h>
#include "army.h"
#include "armyset.h"
#include "player.h"
#include "city.h"
#include <sstream>

using namespace std;

CityInfo::CityInfo(City* city)
	:PG_Window(0, PG_Rect((PG_Application::GetScreenWidth()-350)/2,
                (PG_Application::GetScreenHeight()-250)/2, 350, 250)
            , "City info", WF_MODAL), d_city(city)
{
	char buffer[40];
	sprintf(buffer, "Income: %i, Defense:", d_city->getGold());
	new PG_Label(this, PG_Rect(20, 30, my_width - 20, 30), d_city->getName().c_str());
	new PG_Label(this, PG_Rect(20, 60, 150, 20), buffer);

	sprintf(buffer, "%i", d_city->getDefenseLevel());
	l_defense = new PG_Label(this, PG_Rect(170, 60, 30, 20), buffer);

	// add an empty button to the buttongroup for "NO PRODUCTION"
	b_no_production = new PG_Button(this, 0, PG_Rect(20, 80, 44, 44), "N");
	b_no_production->SetEventObject(MSG_BUTTONCLICK, this,
		       (MSG_CALLBACK_OBJ)&CityInfo::b_production_clicked);
	b_no_production->SetToggle(true);

	for (int i = 0; i < 4; i++)
	{
		int armytype = d_city->getArmytype(i);

		b_production[i] = new PG_Button(this, i+1, PG_Rect(64 + i*44, 80, 44, 44));
		b_production[i]->SetToggle(true);

		if (armytype != -1) 
		{		
        	b_production[i]->SetIcon(d_city->getPlayer()->getArmyset()->
					army[armytype].getPixmap(), 0, 0);
			b_production[i]->SetEventObject(MSG_BUTTONCLICK, this,
				        (MSG_CALLBACK_OBJ)&CityInfo::b_production_clicked);
		}
		else b_production[i]->EnableReceiver(false);
	}

	new PG_Label(this, PG_Rect(20, 130, 80, 20), "Name:");
	new PG_Label(this, PG_Rect(20, 150, 80, 20), "Duration:");
	new PG_Label(this, PG_Rect(20, 170, 80, 20), "Strength:");
	new PG_Label(this, PG_Rect(20, 190, 80, 20), "Moves:");
	new PG_Label(this, PG_Rect(20, 210, 80, 20), "Upkeep:");

	l_production = new PG_Label(this, PG_Rect(120, 130, 80, 20), "0");
	l_duration = new PG_Label(this, PG_Rect(120, 150, 80, 20), "0");
	l_strength = new PG_Label(this, PG_Rect(120, 170, 80, 20), "0");
	l_moves = new PG_Label(this, PG_Rect(120, 190, 80, 20), "0");
	l_upkeep = new PG_Label(this, PG_Rect(120, 210, 80, 20), "0");

	b_upgrade = new PG_Button(this, 1, PG_Rect(my_width - 130, my_height - 110, 110, 30), "Upgrade");
	b_buy_production = new PG_Button(this, 2, PG_Rect(my_width - 130, my_height - 75, 110, 30), "Buy production");
	d_b_close = new PG_Button(this, 3, PG_Rect(my_width - 130, my_height - 40, 110, 30), "Close");

	b_upgrade->SetEventObject(MSG_BUTTONCLICK, this, (MSG_CALLBACK_OBJ)&CityInfo::b_upgrade_clicked);
	b_buy_production->SetEventObject(MSG_BUTTONCLICK, this, (MSG_CALLBACK_OBJ)&CityInfo::b_buy_production_clicked);
	d_b_close->SetEventObject(MSG_BUTTONCLICK, this, (MSG_CALLBACK_OBJ)&CityInfo::d_b_close_clicked);

	// if something is produced put button to DOWN
	if (d_city->getProduction() != -1) b_production[d_city->getProduction()]->SetPressed(true);

	update_production_stats();
	check_buttons();
}

CityInfo::~CityInfo()
{
	delete l_defense;
	delete l_production;
	delete l_duration;
	delete l_strength;
	delete l_moves;
	delete l_upkeep;
	delete b_no_production;
	for (int i = 0; i < 4; i++)
    {
		delete b_production[i];
    }
	delete b_upgrade;
	delete b_buy_production;
	delete d_b_close; 
}

// get the first empty armyproduction-slot of this city

int CityInfo::get_empty_slot()
{
	for (int i = 0; i < 4; i++)
	{
		if (d_city->getArmytype(i) == -1) return i;
	}
	return -1;
}

PARAGUI_CALLBACK(CityInfo::b_production_clicked)
{
	// id is the ID of the button that sent this event
	// id 0 is the "No production" button
	for (int i = 0; i < 4; i++)
	{
		if (i != (id - 1)) b_production[i]->SetPressed(false);
	}

	if (id != 0) b_no_production->SetPressed(false);
		
	d_city->getPlayer()->cityChangeProduction(d_city, id - 1);
	update_production_stats();

	return true;
}

void CityInfo::update_production_stats()
{
	if (d_city->getProduction() >= 0)
	{ 
		// selected armytype
		Armyset_army* a = &d_city->getPlayer()->getArmyset()->
			army[d_city->getArmytype(d_city->getProduction())];

		l_production->SetText(a->getName().c_str());
		char buffer[40];
		sprintf(buffer, "%i", a->getProduction());
		l_duration->SetText(buffer);
		sprintf(buffer, "%i", a->getMaxStrength());
		l_strength->SetText(buffer);
		sprintf(buffer, "%i", a->getMaxMoves());
		l_moves->SetText(buffer);
		sprintf(buffer, "%i", a->getUpkeep());
		l_upkeep->SetText(buffer);
	}
	else
	{
		l_production->SetText("None");
		l_duration->SetText("None");
		l_strength->SetText("None");
		l_moves->SetText("None");
		l_upkeep->SetText("None");
	}
}

PARAGUI_CALLBACK(CityInfo::b_upgrade_clicked)
{
	int gold = needed_gold();

	if ((gold < 0) || (d_city->getPlayer()->getGold() < gold))
	{
		PG_MessageBox mb(GetParent(), PG_Rect(200, 200, 200, 150),
				"Upgrade City",
				"Not enough gold or already reached level 3!",
				PG_Rect(60, 70, 80, 30), "OK");
		mb.Show();
		mb.WaitForClick();
		mb.Hide();
		return true;
	}

	stringstream s;
	string string;

	s <<"Do you want to upgrade\nthe city to level ";
	s << (d_city->getDefenseLevel() + 1) <<" for " <<gold <<" gold?";
	string = s.str();

	PG_Rect rect_1(20, 110, 80, 30);
	PG_Rect rect_2(105, 110, 80, 30);
	PG_MessageBox mb(GetParent(), PG_Rect(200, 200, 200, 150),
		       "Upgrade city", string.c_str(),
		       rect_1, "Yes", rect_2, "No");
	mb.Show();

	if (mb.WaitForClick() == 1)
	{
		d_city->getPlayer()->cityUpgradeDefense(d_city);

		char buffer[40];
		sprintf(buffer, "%i", d_city->getDefenseLevel());
		l_defense->SetText(buffer);
	}
	mb.Hide();

	check_buttons();

	return true;
}

PARAGUI_CALLBACK(CityInfo::b_buy_production_clicked)
{
	D_Buy_Production d_bp(d_city, GetParent(), PG_Rect(20, 20, 340, 250));
	d_bp.Show();
	d_bp.RunModal();

	int chosen_army = d_bp.getChosenArmy();

	if (chosen_army == -1)
    {
		return true;
    }
		
	//the only possible error here is a lack of gold, but this
	//should have already been caught in the selection dialogue
	if (!(d_city->getPlayer()->cityBuyProduction(d_city, chosen_army)))
	{
		cerr <<"Player::city_buy_production returned false ";
		cerr <<"where it shouldn't ";
		cerr <<"(CityInfo::buy_production_clicked)\n";
		return true;
	}

    // slot can have values -2, 0, 1, 2
    // -2 comes from "no empty slots"
    // this means that slot 3 was _just_ filled
	int slot = get_empty_slot() - 1;
 	if (slot == -2)
 	{
 		slot = 3;
 	}
	b_production[slot]->SetIcon(d_city->getPlayer()->getArmyset()->
            army[chosen_army].getPixmap(), 0, 0);
	b_production[slot]->SetEventObject(MSG_BUTTONCLICK, this,
			(MSG_CALLBACK_OBJ)&CityInfo::b_production_clicked);
	b_production[slot]->EnableReceiver(true);
	
	check_buttons();
    Redraw();

	return true;
}

int CityInfo::needed_gold()
{
	if (d_city->getDefenseLevel() == 1) return 1000;
	else if (d_city->getDefenseLevel() == 2) return 2000;
	return -1;
}

void CityInfo::check_buttons()
{
	// Upgrade button
	if ((d_city->getDefenseLevel() < 3 && d_city->getPlayer()->getGold() >= needed_gold()))
	{
		b_upgrade->EnableReceiver(true);
	}
	else b_upgrade->EnableReceiver(false);
	
	// Buy production button
	if (get_empty_slot() != -1) b_buy_production->EnableReceiver(true);
	else b_buy_production->EnableReceiver(false);
}


PARAGUI_CALLBACK(CityInfo::d_b_close_clicked)
{
	SendMessage(this, MSG_MODALQUIT, 0, 0);
	return true;
}

// End of file
