//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "citylist.h"
#include "city.h"
#include "playerlist.h"

//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
#define debug(x)

Citylist* citylist;
Ruinlist* ruinlist;
TempleList* templeList;

Citylist::Citylist()
{
}

Citylist::Citylist(XML_Helper* helper)
{
    //the only two entries are size and version which we ignore for now
    helper->registerTag("city", SigC::slot((*this), &Citylist::load));	
}

Citylist::~Citylist()
{
}

// Number of cities for specific player
int Citylist::countCities(Player* player) const
{
	int cities = 0;
	
	for (const_iterator it = begin(); it != end(); it++)
	{
		if ((*it).getPlayer() == player) cities++;
	}
	
	return cities;
}

void Citylist::nextTurn()
{
	debug("next_turn()");
	for (iterator it = begin(); it != end(); it++)
	{
		(*it).nextTurn();
	}
}

City* Citylist::getNearestEnemyCity(const PG_Point& pos) 
{
    int diff = 9999;
    Citylist::iterator diffit;
	for (Citylist::iterator it = begin(); it != end(); ++it)
	{
	    if ((*it).getPlayer() != playerlist->getActiveplayer())
	    {
    	    PG_Point p = (*it).getPos();
    		int delta = abs(p.x - pos.x) + abs(p.y - pos.y);
    		
    		if (diff > delta)
            {
                diff = delta;
                diffit = it;
    		}
        }
	}
	return &(*diffit);
}

bool Citylist::save(XML_Helper* helper)
{
	bool retval = true;

	retval &= helper->openTag("citylist");
	retval &= helper->saveData("size", size());	//for checking later

	for (iterator it = begin(); it != end(); it++)
		it->save(helper);
	
	retval &= helper->closeTag();

	return retval;
}

bool Citylist::load(std::string tag, XML_Helper* helper)
{
	if (tag != "city")
		return false;
	
	City c(helper);
	push_back(c);

	return true;
}

//------------------------------------------------------------------------------

Ruinlist::Ruinlist()
{
}

Ruinlist::Ruinlist(XML_Helper* helper)
{
    //the only argument up to now is size, which we ignore here
    helper->registerTag("ruin", SigC::slot((*this), &Ruinlist::load));
}

bool Ruinlist::save(XML_Helper* helper)
{
	bool retval = true;

	retval &= helper->openTag("ruinlist");
	retval &= helper->saveData("size", size());

	for (iterator it = begin(); it != end(); it++)
		retval &= it->save(helper);
	
	retval &= helper->closeTag();

	return retval;
}

bool Ruinlist::load(std::string tag, XML_Helper* helper)
{
    if (tag != "ruin")
    //???
        return false;

    Ruin r(helper);
    push_front(r);

    //now we need to set up a callback for the stak residing in the ruin
    helper->registerTag("stack", SigC::slot((*begin()), &Ruin::load));

    return true;
}

//-----------------------------------------------------------------------------

TempleList::TempleList()
{
}

TempleList::TempleList(XML_Helper* helper)
{
	//the only variable of value here is size, and we ignore it for now
	helper->registerTag("temple", SigC::slot((*this), &TempleList::load));

}

bool TempleList::save(XML_Helper* helper)
{
	bool retval = true;

	retval &= helper->openTag("templelist");
	retval &= helper->saveData("size", size());

	for (iterator it = begin(); it != end(); it++)
		retval &= it->save(helper);
	
	retval &= helper->closeTag();

	return retval;
}

bool TempleList::load(std::string tag, XML_Helper* helper)
{
	if (tag != "temple")	
	//what has happened?
		return false;
	
	Temple t(helper);
	push_back(t);

	return true;
}

// End of file
