//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "stacklist.h"
#include "stack.h"
#include "playerlist.h"

using namespace std;

#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
//#define debug(x)

Stacklist* stacklist;

Stacklist::Stacklist()
	:d_activestack(0)
{
}

Stacklist::Stacklist(XML_Helper* helper)
    :d_activestack(0)
{
    helper->registerTag("stack", SigC::slot((*this), &Stacklist::load));
    
    load("stacklist", helper);
}

Stacklist::~Stacklist()
{
}

// Get next Stack of the same player and set it as d_activestack
// current implementation doesn't work correctly
// TODO: implement the following
// 1. Create a vector of all stacks of the current player
// 2. start the iterator with the active stack
// 3. loop through the vector until the active stack is met again

Stack* Stacklist::setNext()
{	
	debug("setNext()");

	// set the iterator to the activestack 
	for (const_iterator it = begin(); it != end(); ++it)
	{
		if ((*it) != d_activestack && (*it)->getPlayer() ==
                playerlist->getActiveplayer() && !(*it)->getDefending())
		{
			d_activestack = *it;
			return d_activestack;
		}
	}
	return 0;
}

void Stacklist::nextTurn()
{
	debug("nextTurn()");

	for (iterator it = begin(); it != end(); it++)
	{
		(*it)->nextTurn();
	}
}

Stack* Stacklist::getObjectAt(int x, int y) const
{
	for (const_iterator it = begin(); it != end(); it++)
	{
		if (((*it)->getPos().x == x) && ((*it)->getPos().y == y))
		{
			return *it;
		}
	}
	return 0;
}

Stack* Stacklist::getObjectAt(PG_Point point) const
{
	return getObjectAt(point.x, point.y);
}

vector<Stack*> Stacklist::defendersInCity(City *city)
{
	debug("defendersInCity()");  

	vector<Stack*> stackvector;
	PG_Point pos = city->getPos();

	for (int i = pos.x; i < pos.x + 2; i++)
	{
		for (int j = pos.y; j < pos.y + 2; j++)
		{
			Stack* stack = getObjectAt(i, j);
			if (stack)
            {
				stackvector.push_back(stack);
            }
		}
	}

	return stackvector;
}

//We only expect one ambiguity at a time with stacks of the same player. This
//never happens except when a stack comes to halt on another stack during
//long movements
Stack* Stacklist::getAmbiguity(Stack* s)
{
	for (iterator it = begin(); it != end(); it++)
		if ((s->getPos().x == (*it)->getPos().x)
			&& (s->getPos().y == (*it)->getPos().y) && (s != *it))
			return (*it);
	
	return 0;
}

PARAGUI_CALLBACK(Stacklist::removeAll)
{
	Player* player = (Player*)clientdata;

	for (iterator it = begin(); it != end(); it++)
	{
        if ((*it)->getPlayer() == player) stacklist->remove(*it);
	}
	return true;
}

void Stacklist::flClear()
{
	for (iterator it = begin(); it != end(); it++)
	{
		delete (*it);
	}
	
	clear();
}

Stacklist::iterator Stacklist::flErase(iterator object)
{
	delete (*object);
	return erase(object);
}

bool Stacklist::flRemove(Stack* object)
{
	debug("removing stack with id " << object->getId() << endl);
	iterator stackit = find(begin(), end(), object);
	if (stackit != end())
	{
		delete object;
		erase(stackit);
		return true;
	}
	return false;
}

bool Stacklist::save(XML_Helper* helper)
{
	bool retval = true;

	retval &= helper->openTag("stacklist");
	retval &= helper->saveData("size", size());
	if (d_activestack)
    {
		retval &= helper->saveData("active", d_activestack->getId());
    }
	else
    {
		retval &= helper->saveData("active", 0);
    }

	//save stacks
	for (iterator it = begin(); it != end(); it++)
		retval &= (*it)->save(helper);
	
	retval &= helper->closeTag();

	return retval;
}

bool Stacklist::load(string tag, XML_Helper* helper)
{
    static unsigned int active = 0;
    if (tag == "stacklist")
    {
        helper->getData(active, "active");
        return true;
    }

    if (tag == "stack")
    {
        Stack* s = new Stack(helper);
        if ((active != 0) && (s->getId() == active))
        {
            d_activestack = s;
        }

        push_back(s);
        return true;
    }

    return false;
}

// End of file
